@@ -5,16 +5,16 @@
    Copyright (C) 2000 Eazel, Inc.
   
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
+   modify it under the terms of the GNU Library General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Library General Public License for more details.
   
-   You should have received a copy of the GNU General Public
+   You should have received a copy of the GNU Library General Public
    License along with this program; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
@@ -26,6 +26,7 @@
 #include "eel-debug-drawing.h"
 
 #include "eel-art-gtk-extensions.h"
+#include "eel-debug.h"
 #include "eel-gdk-extensions.h"
 #include "eel-gdk-extensions.h"
 #include "eel-gdk-pixbuf-extensions.h"
@@ -348,7 +349,7 @@ eel_debug_show_pixbuf (const GdkPixbuf *pixbuf)
 
 		eel_gtk_widget_set_background_color (debug_window, "white");
 
-		g_atexit (destroy_debug_window);
+		eel_debug_call_at_shutdown (destroy_debug_window);
 		
 		gtk_widget_show (debug_image);
 		gtk_widget_show (vbox);
