@@ -30,6 +30,7 @@
 #include "eel-art-extensions.h"
 #include "eel-gdk-extensions.h"
 #include "eel-glib-extensions.h"
+#include "eel-gtk-extensions.h"
 #include "eel-stock-dialogs.h"
 #include <X11/Xatom.h>
 #include <errno.h>
@@ -38,7 +39,7 @@
 #include <gtk/gtkwidget.h>
 #include <libart_lgpl/art_rect.h>
 #include <libart_lgpl/art_rgb.h>
-#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-file-entry.h>
@@ -65,10 +66,8 @@ struct IconSelectionData {
 
 typedef struct IconSelectionData IconSelectionData;
 
-static Atom xa_win_area;
-
 ArtIRect
-eel_gnome_canvas_world_to_window_rectangle (const GnomeCanvas *canvas,
+eel_gnome_canvas_world_to_window_rectangle (GnomeCanvas *canvas,
 					    ArtDRect world_rect)
 {
 	double x0, y0, x1, y1;
@@ -76,11 +75,11 @@ eel_gnome_canvas_world_to_window_rectangle (const GnomeCanvas *canvas,
 
 	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), eel_art_irect_empty);
 
-	gnome_canvas_world_to_window (GNOME_CANVAS (canvas),
+	gnome_canvas_world_to_window (canvas,
 				      world_rect.x0,
 				      world_rect.y0,
 				      &x0, &y0);
-	gnome_canvas_world_to_window (GNOME_CANVAS (canvas),
+	gnome_canvas_world_to_window (canvas,
 				      world_rect.x1,
 				      world_rect.y1,
 				      &x1, &y1);
@@ -94,7 +93,7 @@ eel_gnome_canvas_world_to_window_rectangle (const GnomeCanvas *canvas,
 }
 
 ArtIRect
-eel_gnome_canvas_world_to_canvas_rectangle (const GnomeCanvas *canvas,
+eel_gnome_canvas_world_to_canvas_rectangle (GnomeCanvas *canvas,
 					    ArtDRect world_rect)
 {
 	ArtIRect canvas_rect;
@@ -116,7 +115,7 @@ eel_gnome_canvas_world_to_canvas_rectangle (const GnomeCanvas *canvas,
 }
 
 ArtIRect
-eel_gnome_canvas_item_get_current_canvas_bounds (const GnomeCanvasItem *item)
+eel_gnome_canvas_item_get_current_canvas_bounds (GnomeCanvasItem *item)
 {
 	ArtIRect bounds;
 
@@ -152,13 +151,13 @@ eel_gnome_canvas_request_redraw_rectangle (GnomeCanvas *canvas,
 }
 
 ArtDRect
-eel_gnome_canvas_item_get_world_bounds (const GnomeCanvasItem *item)
+eel_gnome_canvas_item_get_world_bounds (GnomeCanvasItem *item)
 {
 	ArtDRect world_bounds;
 
 	g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (item), eel_art_drect_empty);
 
-	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (item),
+	gnome_canvas_item_get_bounds (item,
 				      &world_bounds.x0,
 				      &world_bounds.y0,
 				      &world_bounds.x1,
@@ -176,7 +175,7 @@ eel_gnome_canvas_item_get_world_bounds (const GnomeCanvasItem *item)
 }
 
 ArtIRect
-eel_gnome_canvas_item_get_canvas_bounds (const GnomeCanvasItem *item)
+eel_gnome_canvas_item_get_canvas_bounds (GnomeCanvasItem *item)
 {
 	ArtDRect world_bounds;
 
@@ -339,22 +338,6 @@ eel_gnome_canvas_fill_rgb (GnomeCanvasBuf *buf, art_u8 r, art_u8 g, art_u8 b)
 	}
 }
 
-GtkButton *
-eel_gnome_dialog_get_button_by_index (GnomeDialog *dialog, int index)
-{
-	gpointer data;
-
-	g_return_val_if_fail (GNOME_IS_DIALOG (dialog), NULL);
-	g_return_val_if_fail (index >= 0, NULL);
-
-	data = g_list_nth_data (GNOME_DIALOG (dialog)->buttons, index);
-	if (data == NULL) {
-		return NULL;
-	}
-
-	return GTK_BUTTON (data);
-}
-
 void
 eel_gnome_canvas_item_request_update_deep (GnomeCanvasItem *item)
 {
@@ -563,7 +546,7 @@ try_terminal_command (const char *program,
 		return NULL;
 	}
 
-	quoted = eel_shell_quote (program_in_path);
+	quoted = g_shell_quote (program_in_path);
 	if (args == NULL || args[0] == '\0') {
 		return quoted;
 	}
@@ -590,7 +573,7 @@ try_terminal_command_argv (int argc,
 	
 	string = g_string_new (argv[1]);
 	for (i = 2; i < argc; i++) {
-		quoted = eel_shell_quote (argv[i]);
+		quoted = g_shell_quote (argv[i]);
 		g_string_append_c (string, ' ');
 		g_string_append (string, quoted);
 		g_free (quoted);
@@ -620,8 +603,11 @@ get_terminal_command_prefix (gboolean for_command)
 	/* Try the terminal from preferences. Use without any
 	 * arguments if we are just doing a standalone terminal.
 	 */
-	gnome_config_get_vector ("/Gnome/Applications/Terminal",
-				 &argc, &argv);
+	argc = 0;
+	argv = g_new0 (char *, 1);
+	gnome_prepend_terminal_to_vector (&argc, &argv);
+
+	command = NULL;
 	if (argc != 0) {
 		if (for_command) {
 			command = try_terminal_command_argv (argc, argv);
@@ -631,21 +617,27 @@ get_terminal_command_prefix (gboolean for_command)
 			 */
 			command = try_terminal_command (argv[0], NULL);
 		}
-		if (command != NULL) {
-			return command;
-		}
+	}
+
+	while (argc != 0) {
+		g_free (argv[--argc]);
+	}
+	g_free (argv);
+
+	if (command != NULL) {
+		return command;
 	}
 
 	/* Try well-known terminal applications in same order that gmc did. */
-	for (i = 0; i < EEL_N_ELEMENTS (commands); i++) {
+	for (i = 0; i < G_N_ELEMENTS (commands); i++) {
 		command = try_terminal_command (commands[i][0],
 						commands[i][for_command ? 1 : 2]);
 		if (command != NULL) {
-			return command;
+			break;
 		}
 	}
 	
-	return NULL;
+	return command;
 }
 
 char *
@@ -657,7 +649,7 @@ eel_gnome_make_terminal_command (const char *command)
 		return get_terminal_command_prefix (FALSE);
 	}
 	prefix = get_terminal_command_prefix (TRUE);
-	quoted = eel_shell_quote (command);
+	quoted = g_shell_quote (command);
 	terminal_command = g_strconcat (prefix, " /bin/sh -c ", quoted, NULL);
 	g_free (prefix);
 	g_free (quoted);
@@ -680,7 +672,6 @@ eel_gnome_open_terminal (const char *command)
 
 /* create a new icon selection dialog */
 
-
 static gboolean
 widget_destroy_callback (gpointer callback_data)
 {
@@ -692,17 +683,17 @@ widget_destroy_callback (gpointer callback_data)
 
 /* set the image of the file object to the selected file */
 static void
-icon_selected_callback (GtkWidget *button, IconSelectionData *selection_data)
+icon_selected_callback (GtkWidget *dialog, IconSelectionData *selection_data)
 {
 	const gchar *icon_path;
-	GtkWidget *entry;
 	struct stat buf;
+	GtkWidget *entry;
 	
 	gnome_icon_selection_stop_loading ( GNOME_ICON_SELECTION (selection_data->icon_selection));
 
 	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (selection_data->file_entry));
 	icon_path = gtk_entry_get_text (GTK_ENTRY (entry));
-
+	
 	/* if a specific file wasn't selected, put up a dialog to tell the
 	 * user to pick something, and leave the picker up
 	 */
@@ -720,13 +711,13 @@ icon_selected_callback (GtkWidget *button, IconSelectionData *selection_data)
 	 * file entry might still need it.  Do it when the next idle rolls around
 	 */ 
 	selection_data->dismissed = TRUE;
-	selection_data->top_level_window = gtk_widget_get_toplevel (button);
+	selection_data->top_level_window = dialog;
 	gtk_idle_add ((GtkFunction) widget_destroy_callback, selection_data);
 }
 
 /* handle the cancel button being pressed */
 static void
-icon_cancel_pressed (GtkWidget *button, IconSelectionData *selection_data)
+icon_cancel_pressed (GtkWidget *dialog, IconSelectionData *selection_data)
 {
 	/* nothing to do if it's already been dismissed */
 	if (selection_data->dismissed) {
@@ -736,7 +727,7 @@ icon_cancel_pressed (GtkWidget *button, IconSelectionData *selection_data)
 	gnome_icon_selection_stop_loading(GNOME_ICON_SELECTION (selection_data->icon_selection));	
 	/* remove pending idle routine, if necessary */
 	g_free (selection_data);
-	gtk_widget_destroy (gtk_widget_get_toplevel (button));
+	gtk_widget_destroy (dialog);
 }
 
 /* handle an icon being selected by updating the file entry */
@@ -759,13 +750,29 @@ list_icon_selected_callback (GnomeIconList *gil, gint num, GdkEvent *event, Icon
 	}
 }
 
+static void
+dialog_response_callback (GtkWidget *dialog, gint response_id, IconSelectionData *selection_data)
+{
+	switch (response_id) {
+	case GTK_RESPONSE_OK:
+		icon_selected_callback (dialog, selection_data);
+		break;
+	case GTK_RESPONSE_CANCEL:
+	case GTK_RESPONSE_DELETE_EVENT:
+		icon_cancel_pressed (dialog, selection_data);
+		break;
+	default:
+		break;
+	}
+}
+
 /* handle the file entry changing */
 static void
 entry_activated (GtkWidget *widget, IconSelectionData *selection_data)
 {
 	struct stat buf;
 	GtkWidget *icon_selection;
-	gchar *filename;
+	const gchar *filename;
 
 	filename = gtk_entry_get_text (GTK_ENTRY (widget));
 	if (!filename) {
@@ -797,13 +804,10 @@ eel_gnome_icon_selector_new (const char *title,
 	GtkWidget *entry, *file_entry;
 	IconSelectionData *selection_data;
 	
-	dialog = gnome_dialog_new (title,
-				   GNOME_STOCK_BUTTON_OK,
-				   GNOME_STOCK_BUTTON_CANCEL,
-				   NULL);
-
-	gnome_dialog_close_hides (GNOME_DIALOG (dialog), TRUE);
-	gnome_dialog_set_close  (GNOME_DIALOG(dialog), TRUE);
+	dialog = gtk_dialog_new_with_buttons (title, owner, 0, 
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
+					      NULL);
 
 	gtk_window_set_policy (GTK_WINDOW (dialog), TRUE, TRUE, TRUE);
 
@@ -818,11 +822,11 @@ eel_gnome_icon_selector_new (const char *title,
 	selection_data->selection_function = selected;
  	selection_data->callback_data = callback_data;
  	
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
-			   file_entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
+			    file_entry, FALSE, FALSE, 0);
 
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
-			   icon_selection, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
+			    icon_selection, TRUE, TRUE, 0);
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 	if (owner != NULL) {
@@ -834,169 +838,133 @@ eel_gnome_icon_selector_new (const char *title,
 	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry));
 	
 	if (icon_directory == NULL) {
-		gtk_entry_set_text(GTK_ENTRY (entry), DATADIR "/pixmaps");
-		gnome_icon_selection_add_directory(GNOME_ICON_SELECTION (icon_selection), DATADIR "/pixmaps");	
+		gtk_entry_set_text (GTK_ENTRY (entry), DATADIR "/pixmaps");
+		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (icon_selection), DATADIR "/pixmaps");	
 	} else {
-		gtk_entry_set_text(GTK_ENTRY (entry), icon_directory);
-		gnome_icon_selection_add_directory(GNOME_ICON_SELECTION (icon_selection), icon_directory);
+		gtk_entry_set_text (GTK_ENTRY (entry), icon_directory);
+		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (icon_selection), icon_directory);
 	}
 	
-	gnome_icon_selection_show_icons( GNOME_ICON_SELECTION (icon_selection));	
-	gnome_dialog_button_connect(GNOME_DIALOG(dialog), 
-				    0, /* OK button */
-				    GTK_SIGNAL_FUNC (icon_selected_callback),
-				    selection_data);
-	
-	gnome_dialog_button_connect (GNOME_DIALOG(dialog), 
-				     1, /* Cancel button */
-				     GTK_SIGNAL_FUNC (icon_cancel_pressed),
-				     selection_data);
-	
-	gtk_signal_connect_after(GTK_OBJECT (GNOME_ICON_SELECTION (selection_data->icon_selection)->gil),
+	gnome_icon_selection_show_icons (GNOME_ICON_SELECTION (icon_selection));
+
+	gtk_signal_connect (GTK_OBJECT (dialog), "response",
+			    G_CALLBACK (dialog_response_callback),
+			    selection_data);
+
+	gtk_signal_connect_after(GTK_OBJECT (gnome_icon_selection_get_gil (GNOME_ICON_SELECTION (selection_data->icon_selection))),
 				 "select_icon",
-				 GTK_SIGNAL_FUNC(list_icon_selected_callback),
+				 G_CALLBACK(list_icon_selected_callback),
 				 selection_data);
 
 	gtk_signal_connect_while_alive( GTK_OBJECT (entry), "activate",
-					GTK_SIGNAL_FUNC(entry_activated),
+					G_CALLBACK(entry_activated),
 					selection_data, GTK_OBJECT (file_entry));
 
 	return dialog;
 }
 
-
-/* Set an icon on GnomeStock widget.  If the setting fails register this
- * as a new file.  Returns FALSE if even that failed */
-gboolean
-eel_gnome_stock_set_icon_or_register (GnomeStock *stock, const char *icon)
+PangoContext *
+eel_gnome_canvas_get_pango_context (GnomeCanvas *canvas)
 {
-	GnomeStockPixmapEntryPath *new_entry;
-
-	g_return_val_if_fail (stock != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_STOCK (stock), FALSE);
-	g_return_val_if_fail (icon != NULL, FALSE);
-
-	/* if we can set the icon and everything goes well we have succeeded */
-	if (gnome_stock_set_icon (stock, icon)) {
-		return TRUE;
+	if (!canvas->aa) {
+		return gtk_widget_get_pango_context (GTK_WIDGET (canvas));
+	} else {
+		return eel_gtk_widget_get_pango_ft2_context (GTK_WIDGET (canvas));
 	}
-
-	/* If the icon string is not an existing file then this is just a
-	 * bogus request and we return FALSE */
-	if ( ! g_file_exists (icon)) {
-		return FALSE;
-	}
-
-	/* If icon exists but gnome_stock_set_icon fails,
-	 * that means this file has NOT been registered with
-	 * gnome stock.  Unfortunately gnome_stock is a
-	 * worthless pile of dung and doesn't do this for us.
-	 * Do note however that it DOES register this stuff
-	 * when it first creates the toolbars from
-	 * GnomeUIInfo.  Go figure.
-	 *
-	 * Note that all this will be completely different in gnome 2.0
-	 */
-
-	new_entry = g_malloc (sizeof (GnomeStockPixmapEntryPath));
-	new_entry->type = GNOME_STOCK_PIXMAP_TYPE_PATH;
-	new_entry->label = NULL;
-	new_entry->pathname = g_strdup (icon);
-	new_entry->width = 0;
-	new_entry->height = 0;
-
-	/* Register this under the "icon" as that's what
-	 * we'll look it up under later.
-	 */
-	gnome_stock_pixmap_register (icon, GNOME_STOCK_PIXMAP_REGULAR,
-				     (GnomeStockPixmapEntry *) new_entry);
-
-	return gnome_stock_set_icon (stock, icon);
-}
-
-static void
-get_win_area (Window xid, int *area_x, int *area_y)
-{
-	Atom actual_type;
-	int actual_format;
-	gulong nitems, bytes_after;
-	gulong *prop;
-
-	if (xa_win_area == 0) {
-		xa_win_area = XInternAtom (GDK_DISPLAY (), "_WIN_AREA", False);
-	}
-
-	gdk_error_trap_push ();
-
-	if (XGetWindowProperty (GDK_DISPLAY (), xid,
-				xa_win_area, 0, 2, False, AnyPropertyType,
-				&actual_type, &actual_format, &nitems,
-				&bytes_after, (guchar **) &prop) == Success
-	    && nitems == 2) {
-		*area_x = prop[0];
-		*area_y = prop[1];
-	}
-
-	gdk_flush ();
-	gdk_error_trap_pop ();
-}
-
-static void
-set_win_area (Window xid, int area_x, int area_y)
-{
-	gulong data[2];
-
-	data[0] = area_x;
-	data[1] = area_y;
-
-	if (xa_win_area == 0) {
-		xa_win_area = XInternAtom (GDK_DISPLAY (), "_WIN_AREA", False);
-	}
-
-	gdk_error_trap_push ();
-
-	XChangeProperty (GDK_DISPLAY (), xid, xa_win_area, XA_CARDINAL,
-			 32, PropModeReplace, (guchar *) data, 2);
-
-	gdk_flush ();
-	gdk_error_trap_pop ();
 }
 
 void
-eel_gnome_win_hints_get_area (GtkWidget *window, int *area_x, int *area_y)
+eel_gnome_canvas_item_send_behind (GnomeCanvasItem *item,
+				   GnomeCanvasItem *behind_item)
 {
-	/* Default to 0,0. Should be mostly harmless */
-	*area_x = 0;
-	*area_y = 0;
+	GList *item_list;
+	int item_position, behind_position;
 
-	g_return_if_fail (GTK_WIDGET_REALIZED (window));
+	g_return_if_fail (GNOME_IS_CANVAS_ITEM (item));
 
-	get_win_area (GDK_WINDOW_XWINDOW (window->window), area_x, area_y);
+	if (behind_item == NULL) {
+		gnome_canvas_item_raise_to_top (item);
+		return;
+	}
+
+	g_return_if_fail (GNOME_IS_CANVAS_ITEM (behind_item));
+	g_return_if_fail (item->parent == behind_item->parent);
+
+	item_list = GNOME_CANVAS_GROUP (item->parent)->item_list;
+
+	item_position = g_list_index (item_list, item);
+	g_assert (item_position != -1);
+	behind_position = g_list_index (item_list, behind_item);
+	g_assert (behind_position != -1);
+	g_assert (item_position != behind_position);
+
+	if (item_position == behind_position - 1) {
+		return;
+	}
+
+	if (item_position < behind_position) {
+		gnome_canvas_item_raise (item, (behind_position - 1) - item_position);
+	} else {
+		gnome_canvas_item_lower (item, item_position - behind_position);
+	}
+}
+
+ArtIRect
+eel_gnome_canvas_world_to_canvas_window_rectangle (GnomeCanvas *canvas,
+						   ArtDRect world_rect)
+{
+	ArtIRect window_rect;
+	
+	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), eel_art_irect_empty);
+
+	eel_gnome_canvas_world_to_canvas_window (canvas,
+						 world_rect.x0, world_rect.y0,
+						 &window_rect.x0, &window_rect.y0);
+	eel_gnome_canvas_world_to_canvas_window (canvas,
+						 world_rect.x1, world_rect.y1,
+						 &window_rect.x1, &window_rect.y1);
+
+	return window_rect;
 }
 
 void
-eel_gnome_win_hints_get_current_area (int *area_x, int *area_y)
+eel_gnome_canvas_canvas_window_to_world (GnomeCanvas *canvas,
+					 int          window_x,
+					 int          window_y,
+					 double      *world_x,
+					 double      *world_y)
 {
-	*area_x = 0;
-	*area_y = 0;
+	int bin_window_x, bin_window_y;
+	
+	g_return_if_fail (GNOME_IS_CANVAS (canvas));
 
-	get_win_area (RootWindow (GDK_DISPLAY (),
-				  DefaultScreen (GDK_DISPLAY ())),
-		      area_x, area_y);
+	bin_window_x = window_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
+	bin_window_y = window_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
+
+	gnome_canvas_window_to_world (canvas,
+				      bin_window_x, bin_window_y,
+				      world_x, world_y);
 }
 
 void
-eel_gnome_win_hints_set_area (GtkWidget *window, int area_x, int area_y)
+eel_gnome_canvas_world_to_canvas_window (GnomeCanvas *canvas,
+					 double       world_x,
+					 double       world_y,
+					 int         *window_x,
+					 int         *window_y)
 {
-	g_return_if_fail (GTK_WIDGET_REALIZED (window));
+	double bin_window_x, bin_window_y;
 
-	set_win_area (GDK_WINDOW_XWINDOW (window->window), area_x, area_y);
-}
+	g_return_if_fail (GNOME_IS_CANVAS (canvas));
 
-void
-eel_gnome_win_hints_set_current_area (int area_x, int area_y)
-{
-	set_win_area (RootWindow (GDK_DISPLAY (),
-				  DefaultScreen (GDK_DISPLAY ())),
-		      area_x, area_y);
+	gnome_canvas_world_to_window (canvas,
+				      world_x, world_y,
+				      &bin_window_x, &bin_window_y);
+
+	if (window_x) {
+		*window_x = (int) bin_window_x - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
+	}
+	if (window_y) {
+		*window_y = (int) bin_window_y - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
+	}
 }
