@@ -137,18 +137,6 @@ eel_gtk_signal_connect_free_data (GtkObject *object,
 		(object, name, func, data, g_free);
 }
 
-/**
- * eel_gtk_window_present:
- * 
- * Presents to the user a window that may be hidden, iconified, or buried.
- * @window: The GtkWindow to be presented to the user.
- **/
-void
-eel_gtk_window_present (GtkWindow *window)
-{
-	gtk_window_present (window);
-}
-
 /**
  * eel_gtk_window_get_geometry_string:
  * @window: a #GtkWindow
@@ -1641,11 +1629,8 @@ static PangoContext *
 create_pango_ft2_context (PangoContext *base_context)
 {
 	PangoContext *context;
-	double dpi_x, dpi_y;
 			
-	dpi_x = gdk_screen_width () * 25.4 / gdk_screen_width_mm ();
-	dpi_y = gdk_screen_height () * 25.4 / gdk_screen_height_mm ();
-	context = pango_ft2_get_context (dpi_x, dpi_y);
+	context = eel_pango_ft2_get_context ();
 
 	pango_context_set_language (context, pango_context_get_language (base_context));
 	pango_context_set_base_dir (context, pango_context_get_base_dir (base_context));
