@@ -31,9 +31,9 @@
 #include "eel-gtk-container.h"
 #include "eel-gtk-extensions.h"
 #include "eel-gtk-macros.h"
-#include "eel-image.h"
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkcheckbutton.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtktogglebutton.h>
 
@@ -300,10 +300,6 @@ eel_labeled_image_get_arg (GtkObject *object,
 
  	switch (arg_id)
 	{
-	case ARG_PIXBUF:
-		GTK_VALUE_POINTER (*arg) = eel_labeled_image_get_pixbuf (labeled_image);
-		break;
-
 	case ARG_LABEL:
 		GTK_VALUE_STRING (*arg) = eel_labeled_image_get_text (labeled_image);
 		break;
@@ -466,7 +462,7 @@ static void
 eel_labeled_image_add (GtkContainer *container,
 		       GtkWidget *child)
 {
-	g_return_if_fail (GTK_IS_LABEL (child) || EEL_IS_IMAGE (child));
+	g_return_if_fail (GTK_IS_LABEL (child) || GTK_IS_IMAGE (child));
 
 	eel_gtk_container_child_add (container, child);
 }
@@ -477,7 +473,7 @@ eel_labeled_image_remove (GtkContainer *container,
 {
 	EelLabeledImage *labeled_image;
 	
-	g_return_if_fail (GTK_IS_LABEL (child) || EEL_IS_IMAGE (child));
+	g_return_if_fail (GTK_IS_LABEL (child) || GTK_IS_IMAGE (child));
 	
 	labeled_image = EEL_LABELED_IMAGE (container);;
 
@@ -1021,7 +1017,7 @@ labeled_image_ensure_image (EelLabeledImage *labeled_image)
 		return;
 	}
 
- 	labeled_image->details->image = eel_image_new (NULL);
+ 	labeled_image->details->image = gtk_image_new ();
 	gtk_container_add (GTK_CONTAINER (labeled_image), labeled_image->details->image);
 	gtk_widget_show (labeled_image->details->image);
 }
@@ -1762,7 +1758,7 @@ eel_labeled_image_set_pixbuf (EelLabeledImage *labeled_image,
 		gtk_widget_queue_resize (GTK_WIDGET (labeled_image));
 	} else {
 		labeled_image_ensure_image (labeled_image);
-		eel_image_set_pixbuf (EEL_IMAGE (labeled_image->details->image), pixbuf);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (labeled_image->details->image), pixbuf);
 	}
 }
 
@@ -1773,19 +1769,7 @@ eel_labeled_image_set_pixbuf_from_file_name (EelLabeledImage *labeled_image,
 	g_return_if_fail (EEL_IS_LABELED_IMAGE (labeled_image));
 
 	labeled_image_ensure_image (labeled_image);
-	eel_image_set_pixbuf_from_file_name (EEL_IMAGE (labeled_image->details->image), pixbuf_file_name);
-}
-
-GdkPixbuf*
-eel_labeled_image_get_pixbuf (const EelLabeledImage *labeled_image)
-{
-	g_return_val_if_fail (EEL_IS_LABELED_IMAGE (labeled_image), NULL);
-
-	if (labeled_image->details->image == NULL) {
-		return NULL;
-	}
-	
-	return eel_image_get_pixbuf (EEL_IMAGE (labeled_image->details->image));
+	gtk_image_set_from_file (GTK_IMAGE (labeled_image->details->image), pixbuf_file_name);
 }
 
 /**
