@@ -4,6 +4,7 @@
                             a single eel preference key.
 
    Copyright (C) 1999, 2000 Eazel, Inc.
+   Copyright (C) 2002 Bent Spoon Software
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -31,7 +32,6 @@
 #include "eel-font-picker.h"
 #include "eel-glib-extensions.h"
 #include "eel-gtk-extensions.h"
-#include "eel-gtk-macros.h"
 #include "eel-marshal.h"
 #include "eel-preferences.h"
 #include "eel-radio-button-group.h"
@@ -42,6 +42,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <libgnome/gnome-macros.h>
 
 #define PREFERENCES_ITEM_UNDEFINED_ITEM -1U
 
@@ -73,90 +74,27 @@ typedef enum
 	LAST_SIGNAL
 } EelStringListSignals;
 
-static guint preferences_item_signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL];
 
-static void eel_preferences_item_class_init                     (EelPreferencesItemClass *preferences_item_class);
-static void eel_preferences_item_init                           (EelPreferencesItem      *preferences_item);
-static void preferences_item_finalize                                 (GObject               *object);
+static void preferences_item_update_displayed_value                   (EelPreferencesItem  *item);
+static void preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem  *item);
+static void preferences_item_update_text_settings_at_idle             (EelPreferencesItem  *item);
+static void preferences_item_update_description                       (EelPreferencesItem  *item);
+static void enumeration_radio_changed_callback                        (EelRadioButtonGroup *radio_button_group,
+								       gpointer             callback_data);
+static void boolean_button_toggled_callback                           (GtkWidget           *button_group,
+								       gpointer             user_data);
+static void editable_string_changed_callback                          (GtkWidget           *caption,
+								       gpointer             user_data);
+static void editable_integer_changed_callback                         (GtkWidget           *caption,
+								       gpointer             user_data);
+static void enumeration_menu_changed_callback                         (EelStringPicker     *string_picker,
+								       EelPreferencesItem  *item);
+static void enumeration_list_changed_callback                         (EelStringPicker     *string_picker,
+								       EelPreferencesItem  *item);
 
-/* Private stuff */
-static void preferences_item_create_boolean                           (EelPreferencesItem      *item);
-static void preferences_item_create_editable_integer                  (EelPreferencesItem      *item);
-static void preferences_item_create_editable_string                   (EelPreferencesItem      *item);
-static void preferences_item_create_enumeration_menu                  (EelPreferencesItem      *item);
-static void preferences_item_create_enumeration_radio                 (EelPreferencesItem      *item,
-								       gboolean                 horizontal);
-static void preferences_item_create_enumeration_list                  (EelPreferencesItem      *item,
-								       gboolean                 horizontal);
-static void preferences_item_create_font                              (EelPreferencesItem      *item);
-static void preferences_item_create_padding                           (EelPreferencesItem      *item);
-static void preferences_item_create_smooth_font                       (EelPreferencesItem      *item);
-static void preferences_item_update_displayed_value                   (EelPreferencesItem      *preferences_item);
-static void preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem      *preferences_item);
-static void preferences_item_update_text_settings_at_idle             (EelPreferencesItem      *preferences_item);
-static void preferences_item_update_description                       (EelPreferencesItem      *preferences_item);
-
-/* User triggered item changed callbacks */
-static void enumeration_radio_changed_callback                        (EelRadioButtonGroup     *radio_button_group,
-								       gpointer                 callback_data);
-static void boolean_button_toggled_callback                           (GtkWidget               *button_group,
-								       gpointer                 user_data);
-static void editable_string_changed_callback                          (GtkWidget               *caption,
-								       gpointer                 user_data);
-static void editable_integer_changed_callback                         (GtkWidget               *caption,
-								       gpointer                 user_data);
-static void enumeration_menu_changed_callback                         (EelStringPicker         *string_picker,
-								       EelPreferencesItem      *item);
-static void font_changed_callback                                     (GtkWidget               *caption,
-								       gpointer                 user_data);
-static void enumeration_list_changed_callback                         (EelStringPicker         *string_picker,
-								       EelPreferencesItem      *item);
-static void smooth_font_changed_callback                              (EelFontPicker           *font_picker,
-								       gpointer                 callback_data);
-static void custom_changed_callback                                   (GtkWidget               *custom,
-								       gpointer                 user_data);
-
-EEL_CLASS_BOILERPLATE (EelPreferencesItem, eel_preferences_item, GTK_TYPE_VBOX)
-
-/* EelPreferencesItemClass methods */
-static void
-eel_preferences_item_class_init (EelPreferencesItemClass *preferences_item_class)
-{
-	GtkObjectClass *object_class;
-
-	object_class = GTK_OBJECT_CLASS (preferences_item_class);
-
-	/* GtkObjectClass */
-	G_OBJECT_CLASS (preferences_item_class)->finalize = preferences_item_finalize;
-
-	/* Signals */
-	preferences_item_signals[CUSTOM_UPDATE_DISPLAYED_VALUE] = 
-		gtk_signal_new ("custom_update_displayed_value",
-				GTK_RUN_LAST,
-				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EelPreferencesItemClass, custom_update_displayed_value),
-				gtk_marshal_VOID__VOID,
-				G_TYPE_NONE, 
-				0);
-	preferences_item_signals[CUSTOM_DESCRIPTION_CHANGED] = 
- 		gtk_signal_new ("custom_description_changed",
- 				GTK_RUN_LAST,
- 				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EelPreferencesItemClass, custom_update_description),
-				eel_marshal_VOID__POINTER_STRING,
- 				G_TYPE_NONE, 
- 				2,
- 				GTK_TYPE_OBJECT,
- 				GTK_TYPE_STRING);
-}
-
-static void
-eel_preferences_item_init (EelPreferencesItem *item)
-{
-	item->details = g_new0 (EelPreferencesItemDetails, 1);
-	item->details->item_type = PREFERENCES_ITEM_UNDEFINED_ITEM;
-	item->details->enumeration_list_unique_exceptions = eel_string_list_new (TRUE);
-}
+GNOME_CLASS_BOILERPLATE (EelPreferencesItem, eel_preferences_item,
+			 GtkVBox, GTK_TYPE_VBOX)
 
 /* GtkObjectClass methods */
 static void
@@ -172,7 +110,7 @@ preferences_item_finalize (GObject *object)
 	eel_string_list_free (item->details->enumeration_list_unique_exceptions);
 	g_free (item->details);
 
-	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	G_OBJECT_CLASS(object)->finalize (object);
 }
 
 /*
@@ -330,7 +268,7 @@ preferences_item_update_custom (EelPreferencesItem *item)
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
 	gtk_signal_emit (GTK_OBJECT (item),
-			 preferences_item_signals[CUSTOM_UPDATE_DISPLAYED_VALUE]);
+			 signals[CUSTOM_UPDATE_DISPLAYED_VALUE]);
 }
 
 /* This callback is called whenever the preference value changes, so that we can
@@ -724,77 +662,6 @@ preferences_item_create_enumeration_menu (EelPreferencesItem *item)
 	preferences_item_set_main_child (item, child);
 }
 
-static void
-preferences_item_update_font (EelPreferencesItem *item)
-{
-	char *current_value;
-
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_FONT);
-
-	current_value = eel_preferences_get (item->details->preference_name);
-	g_assert (current_value != NULL);
-
-	/* The value of the gconf preference can be anything.  In theory garbage could
-	 * be used for the preference using a third party tool.  So we make sure that
-	 * it is one of the choice before trying to select it, otherwise we would get
-	 * assertions.
-	 */
-	if (eel_string_picker_contains (EEL_STRING_PICKER (item->details->main_child), current_value)) {
-		eel_string_picker_set_selected_string (EEL_STRING_PICKER (item->details->main_child),
-						       current_value);
-	}
-
-	g_free (current_value);
-}
-
-static void
-preferences_item_create_font (EelPreferencesItem *item)
-{
-	EelStringList *font_list;
-	GtkWidget *child;
-
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
-
-	child = eel_string_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), "");
-	
-	/* FIXME bugzilla.eazel.com 1274: Need to query system for available fonts */
-	font_list = eel_string_list_new (TRUE);
-
-	/* Once upon a time we had a bug in Eel that caused crashes with the "fixed" 
-	 * font.  That bug (2256) was fixed by removing the "fixed" choice from this menu
-	 * below.  Subsequently we fixed many font bugs in eel (such hard coded font sizes)
-	 * that would cause both crashes and ugliness. Bug 2256 seems to have been fixed by
-	 * these changes as well.
-	 *
-	 * Anyhow, the "fixed" font choice is not very interesting because the other fonts 
-	 * look much better.  However, in multi byte locales, the fixed font is usually the
-	 * only one that is available at the right encoding.
-	 */
-
-	/* FIXME bugzilla.eazel.com 7907: 
-	 * The "GTK System Font" string is hard coded in many places.
-	 */
-	eel_string_list_insert (font_list, "GTK System Font");
-	eel_string_list_insert (font_list, "fixed");
-	eel_string_list_insert (font_list, "helvetica");
-	eel_string_list_insert (font_list, "times");
-	eel_string_list_insert (font_list, "courier");
-	eel_string_list_insert (font_list, "lucida");
-
-	eel_string_picker_set_string_list (EEL_STRING_PICKER (child), font_list);
-	eel_string_list_free (font_list);
-	
-	preferences_item_add_connection_child (item,
-					       child,
-					       "changed",
-					       G_CALLBACK (font_changed_callback));
-	
-	preferences_item_set_main_child (item, child);
-}
-
 static void
 preferences_item_create_padding (EelPreferencesItem *item)
 {
@@ -804,24 +671,23 @@ preferences_item_create_padding (EelPreferencesItem *item)
 }
 
 static void
-preferences_item_update_smooth_font (EelPreferencesItem *item)
+preferences_item_update_font (EelPreferencesItem *item)
 {
  	char *current_value;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_SMOOTH_FONT);
+	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_FONT);
 
  	current_value = eel_preferences_get (item->details->preference_name);
  	g_assert (current_value != NULL);
-
  	eel_font_picker_set_selected_font (EEL_FONT_PICKER (item->details->main_child),
 					   current_value);
  	g_free (current_value);
 }
 
 static void
-smooth_font_changed_callback (EelFontPicker *font_picker,
-			      gpointer callback_data)
+font_changed_callback (EelFontPicker *font_picker,
+		       gpointer callback_data)
 {
 	EelPreferencesItem *item;
  	char *new_value;
@@ -837,7 +703,7 @@ smooth_font_changed_callback (EelFontPicker *font_picker,
 }
 
 static void
-preferences_item_create_smooth_font (EelPreferencesItem *item)
+preferences_item_create_font (EelPreferencesItem *item)
 {
 	GtkWidget *child;
 
@@ -847,10 +713,8 @@ preferences_item_create_smooth_font (EelPreferencesItem *item)
 	child = eel_font_picker_new (NULL);
 	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
-	preferences_item_add_connection_child (item,
-					       child,
-					       "changed",
-					       G_CALLBACK (smooth_font_changed_callback));
+	preferences_item_add_connection_child (item, child, "changed",
+					       G_CALLBACK (font_changed_callback));
 
 	preferences_item_set_main_child (item, child);
 }
@@ -871,7 +735,7 @@ eel_preferences_item_new (const char *preference_name,
 
 	g_return_val_if_fail (eel_strlen (preference_name) > 0, NULL);
 	g_return_val_if_fail (item_type >= EEL_PREFERENCE_ITEM_BOOLEAN, FALSE);
-	g_return_val_if_fail (item_type <= EEL_PREFERENCE_ITEM_SMOOTH_FONT, FALSE);
+	g_return_val_if_fail (item_type <= EEL_PREFERENCE_ITEM_PADDING, FALSE);
 
 	item = EEL_PREFERENCES_ITEM
 		(gtk_widget_new (eel_preferences_item_get_type (), NULL));
@@ -906,10 +770,6 @@ eel_preferences_item_new (const char *preference_name,
 		preferences_item_create_font (item);
 		break;
 	
-	case EEL_PREFERENCE_ITEM_SMOOTH_FONT:
-		preferences_item_create_smooth_font (item);
-		break;
-	
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 		preferences_item_create_editable_string (item);
 		break;	
@@ -1007,27 +867,6 @@ boolean_button_toggled_callback (GtkWidget *button, gpointer user_data)
 	eel_preferences_set_boolean (item->details->preference_name, active_state);
 }
 
-static void
-font_changed_callback (GtkWidget *string_picker, gpointer user_data)
-{
-	EelPreferencesItem *item;
-	char *selected_string;
-
-	g_return_if_fail (EEL_IS_STRING_PICKER (string_picker));
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (user_data));
-
-	item = EEL_PREFERENCES_ITEM (user_data);
-
-	g_return_if_fail (item->details->preference_name != NULL);
-
-	selected_string = eel_string_picker_get_selected_string (EEL_STRING_PICKER (string_picker));
-	g_return_if_fail (selected_string != NULL);
-
-	eel_preferences_set (item->details->preference_name, selected_string);
-
-	g_free (selected_string);
-}
-
 static void
 editable_string_changed_callback (GtkWidget *button, gpointer user_data)
 {
@@ -1198,10 +1037,6 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 		preferences_item_update_font (item);
 		break;
 	
-	case EEL_PREFERENCE_ITEM_SMOOTH_FONT:
-		preferences_item_update_smooth_font (item);
-		break;
-	
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 		preferences_item_update_editable_string (item);
 		break;	
@@ -1478,7 +1313,6 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 				    description);
 		break;
 	case EEL_PREFERENCE_ITEM_FONT:
-	case EEL_PREFERENCE_ITEM_SMOOTH_FONT:
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
@@ -1491,7 +1325,7 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 		break;
 	case EEL_PREFERENCE_ITEM_CUSTOM:
 		gtk_signal_emit (GTK_OBJECT (item),
-				 preferences_item_signals[CUSTOM_DESCRIPTION_CHANGED],
+				 signals[CUSTOM_DESCRIPTION_CHANGED],
 				 item->details->main_child,
 				 description);
 		break;
@@ -1499,3 +1333,37 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 		g_assert_not_reached ();
 	}
 }
+
+static void
+eel_preferences_item_class_init (EelPreferencesItemClass *class)
+{
+	G_OBJECT_CLASS (class)->finalize = preferences_item_finalize;
+
+	/* Signals */
+	signals[CUSTOM_UPDATE_DISPLAYED_VALUE] = 
+		gtk_signal_new ("custom_update_displayed_value",
+				GTK_RUN_LAST,
+				GTK_CLASS_TYPE (class),
+				GTK_SIGNAL_OFFSET (EelPreferencesItemClass, custom_update_displayed_value),
+				gtk_marshal_VOID__VOID,
+				G_TYPE_NONE, 
+				0);
+	signals[CUSTOM_DESCRIPTION_CHANGED] = 
+ 		gtk_signal_new ("custom_description_changed",
+ 				GTK_RUN_LAST,
+ 				GTK_CLASS_TYPE (class),
+				GTK_SIGNAL_OFFSET (EelPreferencesItemClass, custom_update_description),
+				eel_marshal_VOID__POINTER_STRING,
+ 				G_TYPE_NONE, 
+ 				2,
+ 				GTK_TYPE_OBJECT,
+ 				GTK_TYPE_STRING);
+}
+
+static void
+eel_preferences_item_instance_init (EelPreferencesItem *item)
+{
+	item->details = g_new0 (EelPreferencesItemDetails, 1);
+	item->details->item_type = PREFERENCES_ITEM_UNDEFINED_ITEM;
+	item->details->enumeration_list_unique_exceptions = eel_string_list_new (TRUE);
+}
